#include "states/GameState.h"
#include "states/MainMenuState.h"
#include "states/InGameState.h"
#include "states/EndingState.h"
#include "Renderer.h"
#include <SDL2/SDL.h>
#include <memory>
#include <unordered_map>
#include <iostream>

int main() {
    TTF_Init();

    SDL_Window* window = SDL_CreateWindow("Obstacle Game",
                                          SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
                                          WINDOW_WIDTH, WINDOW_HEIGHT, SDL_WINDOW_SHOWN);
    SDL_Renderer* renderer = SDL_CreateRenderer(window, -1, SDL_RENDERER_SOFTWARE);

    std::unordered_map<std::string, std::shared_ptr<GameState>> states;
    states["main_menu"] = std::make_shared<MainMenuState>();
    states["game"] = std::make_shared<InGameState>();
    states["ending"] = std::make_shared<EndingState>();

    std::shared_ptr<GameState> currentState = states["main_menu"];

    Uint64 NOW = SDL_GetPerformanceCounter();
    Uint64 LAST = 0;
    double deltaTime = 0;

    currentState->init();

    bool running = true;
    while (running) {
        LAST = NOW;
        NOW = SDL_GetPerformanceCounter();

        deltaTime = (double)((NOW - LAST)*1000 / (double)SDL_GetPerformanceFrequency() );

        SDL_Event event;
        while (SDL_PollEvent(&event)) {
            if (event.type == SDL_QUIT) {
                running = false;
            }
            currentState->handleEvents(event);
        }

        currentState->update();
        currentState->render(renderer, deltaTime);

        if (!currentState->getNextState().empty()) {
            std::string nextStateName = currentState->getNextState();
            if (nextStateName == "exit") {
                running = false;
            } else if (states.find(nextStateName) != states.end()) {
                currentState = states[nextStateName];
                currentState->init();
            }
            currentState->clearNextState();
        }
    }

    SDL_DestroyRenderer(renderer);
    SDL_DestroyWindow(window);
    SDL_Quit();
    TTF_Quit();

    return 0;
}
